###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: actionfix.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: actionfix
# version: 1.2.2.0
# description: Adds filt bindings that change mIRC dcc chat actions to .me, \
#	and /me to .me for telnet users.

namespace eval actionfix {

  ##
  ## Fix mIRC dcc chat /me messages
  ##
  ## Args:    idx, text
  ## Returns: processed text
  ##
  proc filt:dccAction {idx text} {
    return ".me [string trim [join [lrange [split $text] 1 end]] \001]"
  }

  ##
  ## Fix telnet session /me messages
  ##
  ## Args:    idx, text
  ## Returns: processed text
  ##
  proc filt:telnetAction {idx text} {
    return ".me [join [lrange [split $text] 1 end]]"
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what filt
    }
    foreach type $what {
      switch -exact -- $type {
        filt {
          $action filt -|- "\001ACTION *\001" ${NamespaceCurrent}::filt:dccAction
          $action filt -|- "/me *" ${NamespaceCurrent}::filt:telnetAction
        }
      }
    }
    return
  }

} ;# namespace actionfix

return
