###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: botping.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: botping
# version: 1.5.0.0
# description: Adds a dcc command called `.ping' and botnet support for \
#	sending/replying to ping requests from other bots.
# requires: isNumber duration calibrateClicksPerSecond clicksPerSecond

namespace eval botping {

  ##
  ## Send ping request to a bot or the botnet: dcc
  ##
  ## Args:    handle, idx, botname
  ## Returns: 1
  ##
  proc dcc:ping {hand idx arg} {
    variable DataIdx

    set bot [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $bot]} then {
      set DataIdx($hand) $idx
      calibrateClicksPerSecond
      putallbots "ping $hand [clock seconds] [clock clicks]"
    } elseif {[islinked $bot]} then {
      set DataIdx($hand) $idx
      calibrateClicksPerSecond
      putbot $bot "ping $hand [clock seconds] [clock clicks]"
    } else {
      putdcc $idx "Error: $bot is not connected to the botnet."
    }
    return 1
  }

  ##
  ## Reply to a ping request on the botnet: bot
  ##
  ## Args:    bot, command, arg
  ## Returns: 0
  ##
  proc bot:ping {bot cmd arg} {
    putbot $bot "pong $arg"
    return 0
  }

  ##
  ## Display ping replys from bots: bot
  ##
  ## Args:    bot, command, arg
  ## Returns: 0
  ##
  proc bot:pong {bot cmd arg} {
    variable DataIdx

    set now "[clock seconds] [clock clicks]"
    set arg [split $arg]
    set hand [lindex $arg 0]
    set seconds [lindex $arg 1]
    if {([validuser $hand]) && ([info exists DataIdx($hand)]) && \
        ([isNumber $seconds])} then {
      if {([set time [expr [lindex $now 0] - $seconds]] < 60) && \
          ([isNumber [set clicks [lindex $arg 2]]]) && \
          ([set clicktime [expr (([lindex $now 1] - $clicks) + 0.0) / [clicksPerSecond]]] >= 0)} then {
        putdcc $DataIdx($hand) "Ping reply from $bot: [format "%f" $clicktime] seconds"
      } else {
        putdcc $DataIdx($hand) "Ping reply from $bot: [duration $time]"
      }
    }
    return 0
  }

  ##
  ## Clear invalid idx data from DataIdx array
  ##
  ## Args:    minute, hour, day, month, year
  ## Returns: 0
  ##
  proc time:hourly {minute hour day month year} {
    variable DataIdx

    if {[string compare "" [set tmp [array get DataIdx]]]} then {
      foreach {hand idx} $tmp {
        if {(![valididx $idx]) || \
            ([string compare [string tolower $hand] \
              [string tolower [idx2hand $idx]]])} then {
          catch {
            unset DataIdx($hand)
          }
        }
      }
    }
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "dcc bot time"
    }
    foreach type $what {
      switch -exact -- $type {
        dcc {
          $action dcc mt|- ping ${NamespaceCurrent}::dcc:ping
        }
        bot {
          $action bot -|- ping ${NamespaceCurrent}::bot:ping
          $action bot -|- pong ${NamespaceCurrent}::bot:pong
        }
        time {
          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:hourly
        }
      }
    }
    return
  }

} ;# namespace botping

return
