
# name: misctools
# version: 0.1.0.0
# config: misctools.conf
# description:
# requires: this_module_needs_to_be_converted

############################################################

  # return user@host of user running this code
  proc contact_address {} {
    global tcl_platform
    variable conf
    if {[string compare "" $conf(email)]} then {
      return $conf(email)
    } elseif {[info exists tcl_platform(user)]} then {
      return "$tcl_platform(user)@[info hostname]"
    }
    return ""
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what pub
    }
    foreach i $what {
      switch -exact -- $i {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}help ${NamespaceCurrent}::pub:help
        }
        msg {
          $action msg -|- help ${NamespaceCurrent}::msg:help
        }
        dcc {
          $action dcc -|- help ${NamespaceCurrent}::dcc:help
        }
        fil {
          $action fil -|- help ${NamespaceCurrent}::fil:help
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    email ""
  }

  # Configuration data descriptions
  array set configDataDesc {
    email "\[address\] email address of user running this script"
  }


############################################################

  # help engine
  proc wphelp {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable NamespaceParent
    set what [lindex [split $arg] 0]
    if {(![string compare "" $what]) || (![string compare help $what])} then {
      foreach i [lsort [array names ${NamespaceParent}::mod]] {
        foreach j [join [lsort [array names ${NamespaceParent}::${i}::help_${type}]]] {
          set flags [lindex [set ${NamespaceParent}::${i}::help_${type}($j)] 2]
          if {[validuser $hand]} then {
            if {[matchattr $hand $flags]} then {
              lappend list($i) $j
            } elseif {((![string compare pub $type]) || \
                       (![string compare dcc $type])) && \
                      ((([validchan $chan]) && \
                        ([matchattr $hand $flags $chan])) || \
                       ([find_chan_flag $hand $flags]))} then {
              lappend list($i) $j
            }
          } elseif {![string compare "-" [lindex [split $flags |] 0]]} then {
            lappend list($i) $j
          }
        }
      }
      if {[llength [array names list]] >= 1} then {
        put $resp2 $dest2 "Wolfpack module help:"
        foreach i [lsort [array names list]] {
          put $resp2 $dest2 "Module $i:"
          formatColumns $resp2 $dest2 $list($i) 5 13 "  " "  "
        }
      } else {
        return 0
      }
    } else {
      foreach i [lsort [array names ${NamespaceParent}::mod]] {
        set topic [array_find ${NamespaceParent}::${i}::help_${type} $what]
        set ok 0
        if {[string compare "" $topic]} then {
          set flags [lindex [set ${NamespaceParent}::${i}::help_${type}($topic)] 2]
          if {[validuser $hand]} then {
            if {[matchattr $hand $flags]} then {
              set ok 1
              break
            } elseif {((![string compare pub $type]) || \
                       (![string compare dcc $type])) && \
                      ((([validchan $chan]) && \
                        ([matchattr $hand $flags $chan])) || \
                       ([find_chan_flag $hand $flags]))} then {
              set ok 1
              break
            }
          } elseif {![string compare "-" [lindex [split $flags |] 0]]} then {
            set ok 1
            break
          }
        }
      }
      if {$ok} then {
        put $resp2 $dest2 "###  $topic [lindex [set ${NamespaceParent}::${i}::help_${type}($topic)] 3]"
        formatLists $resp2 $dest2 [lindex [set ${NamespaceParent}::${i}::help_${type}($topic)] 4] 60 " " " " " " "   " "   "
      } else {
        return 0
      }
    }
    return 1
  }

  # help frontend; pub
  proc pub:help {nick uhost hand chan arg} {
    return [help pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # help frontend; msg
  proc msg:help {nick uhost hand arg} {
    return [help msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # help frontend; dcc
  proc dcc:help {hand idx arg} {
    return [help dcc putdcc $idx putdcc $idx "" "" $hand [lindex [console $idx] 0] $arg]
  }

  # help frontend; fil
  proc fil:help {hand idx arg} {
    return [help fil putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # help frontend
  proc help {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[validHandle $hand]} then {
      set arg [split [string trimright $arg]]
      set mode 0
      set fallthru 0
      set return 1
      switch -regexp [lindex $arg 0] {
        (^$|^help$) {
          set mode 0
          set what $arg
        }
        (^eggdrop$) {
          set mode 1
          set what [join [lrange $arg 1 end]]
        }
        (^wolfpack$) {
          set mode 2
          set what [join [lrange $arg 1 end]]
        }
        default {
          set mode 3
          set what $arg
        }
      }
      if {$mode == 3} then {
        if {![wphelp $type $resp1 $dest1 $resp2 $dest2 $nick $uhost $hand $chan $what]} then {
          set fallthru 1
        }
      }
      if {($mode == 0) || ($mode == 1) || ($fallthru)} then {
        switch -exact $type {
          msg {
            *msg:help $nick $uhost $hand $what
          }
          dcc {
            *dcc:help $hand $dest2 $what
            set return 0
          }
          fil {
            *fil:help $hand $dest2 $what
            set return 0
          }
          pub {
            if {$mode == 1} then {
              put $resp2 $dest2 "No help avaliable on that."
            }
          }
        }
      }
      if {($mode == 0) || ($mode == 2)} then {
        if {![wphelp $type $resp1 $dest1 $resp2 $dest2 $nick $uhost $hand $chan $what]} then {
          put $resp2 $dest2 "No help available on that."
        }
      }
      return $return
    }
    return 0
  }

  # load help data from the given file into the help arrays
  proc _load_help {module} {
    variable conf
    set helpfile [file join $conf(helppath) ${module}.help]
    if {[file exists $helpfile]} then {
      if {[catch {set fd [open $helpfile r]} result]} then {
        module_log o * "Error: unable to load help file `[file tail $helpfile]': $result"
      } else {
        while {![eof $fd]} {
          gets $fd i
          set type [lindex [lindex $i 1] 1]
          if {[string compare "" $i]} then {
            set ::wp::${module}::help_${type}([lindex $i 0]) [join [lrange $i 1 end]]
          } else {
            break
          }
        }
        close $fd
      }
    }
    return
  }

############################################################
