###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: moonphase.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: moonphase
# version: 1.6.6.3
# config: moonphase.conf
# description:
# requires: this_module_needs_to_be_converted

namespace eval moonphase {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  # set data information
  set data(moondata) "1 $conf(moondatafile) ${NamespaceCurrent}::moondata"

  # load data
  module_data load "" 1 0 "1 $conf(moondatafile) ${NamespaceCurrent}::moondata"

  # show moon data; pub
  proc pub:moon {nick uhost hand chan arg} {
    return [moon pub putmsg $chan putnot $nick $nick $uhost $hand $chan $arg]
  }

  # show moon data; msg
  proc msg:moon {nick uhost hand arg} {
    return [moon msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # show moon data; dcc
  proc dcc:moon {hand idx arg} {
    return [moon dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # show moon data
  proc moon {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable moondata
    if {[validHandle $hand]} then {
      if {(![string compare none $moondata(face)]) && \
          (![string compare none $moondata(phase)])} then {
        put $resp1 $dest1 "Phase of the Moon for $moondata(date): Retrieving Data..."
      } elseif {$moondata(found)} then {
        put $resp1 $dest1 "Phase of the Moon for $moondata(date): $moondata(face) with $moondata(percent) of the Moon's surface illuminated."
      } else {
        put $resp1 $dest1 "Phase of the Moon for $moondata(date): $moondata(phase)"
        if {(![string compare "Full Moon" $moondata(phase)]) && \
            ([string compare "" $chan])} then {
#FIXME: redo
#          catch {
#            sound:send $chan "[sound:random howl] howls at the full moon"
#          }
        }
      }
      return 1
    }
    return 0
  }

  # get moon data; pub
  proc pub:getmoon {nick uhost hand chan arg} {
    if {[auth_check pub $nick $uhost $hand $chan getmoon]} then {
      putnot $nick "Retrieving moon phase data..."
      getmoon
      return 1
    }
    return 0
  }

  # get moon data; msg
  proc msg:getmoon {nick uhost hand arg} {
    if {[auth_check pub $nick $uhost $hand $chan getmoon]} then {
      putnot $nick "Retrieving moon phase data..."
      getmoon
      return 1
    }
    return 0
  }

  # get moon data; dcc
  proc dcc:getmoon {hand idx arg} {
    putdcc $idx "Retrieving moon phase data..."
    getmoon
    return 1
  }

  # get moon data
  proc getmoon {} {
    variable conf
    variable moondata
    variable NamespaceParent
    variable NamespaceCurrent
    set time [clock format [clock seconds] -format "%d %m %Y"]
    set day [lindex $time 0]
    set month [lindex $time 1]
    set year [lindex $time 2]
    set content(string) "FFX=2&ID=WP&y=$year&m=$month&d=$day&xx0=$conf(xx0)&xx1=$conf(xx1)&xx2=$conf(xx2)&yy0=$conf(yy0)&yy1=$conf(yy1)&yy2=$conf(yy2)&zz0=$conf(zz0)&zz1=$conf(zz1)&place=$conf(place)&ZZZ=END"
    regsub -nocase -all -- " " $content(string) "+" content(string)
    set content(length) [string length $content(string)]
    set ctime [ctime [clock seconds]]
    set moondata(date) "[lrange $ctime 0 2] [lindex $ctime 4]"
    set moondata(found) 0
    set moondata(face) none
    set moondata(percent) 0%
    set moondata(phase) none
    if {$conf(debug)} then {
      set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] w]
      close $fd
    }
    if {[catch {set idx [connect $conf(host) $conf(port)]} result]} then {
      wpLog o * $NamespaceCurrent "Error: unable to connect to $conf(host):$conf(port)': $result"
    } else {
      control $idx ${NamespaceCurrent}::control_getmoon
      putdcc $idx "POST $conf(path) HTTP/1.0"
      putdcc $idx "Pragma: no-cache"
      putdcc $idx "From: [contact_address]"
      putdcc $idx "User-Agent: wolfpack.tcl\[moonphase.tcl\]/ver"
      putdcc $idx "Content-Length: $content(length)"
      putdcc $idx ""
      putdcc $idx "$content(string)"
      putdcc $idx ""
      return 1
    }
    return 0
  }

  # grab the data we want
  proc control_getmoon {idx text} {
    variable conf
    variable moondata
    variable moondata_changed
    variable NamespaceParent
    set strip "<p> </p> &nbsp;"
    set newtext [replaceExpr [replaceExpr $text $strip] "  " " "]
    if {(!$moondata(found)) && \
        ([regexp ".*Phase of the Moon.*" $newtext])} then {
      set moondata(face) [lrange $newtext 7 8]
      set moondata(found) 1
      set moondata_changed 1
    }
    if {($moondata(found)) && \
        ([regexp ".*% of the Moon.*" $newtext])} then {
      set moondata(percent) [lindex $newtext 1]
      set moondata_changed 1
    }
    if {(!$moondata(found)) && \
        ([regexp ".*New Moon.*" $newtext])} then {
      set moondata(phase) "New Moon"
      set moondata_changed 1
    }
    if {(!$moondata(found)) && \
        ([regexp ".*First quarter.*" $newtext])} then {
      set moondata(phase) "First quarter"
      set moondata_changed 1
    }
    if {(!$moondata(found)) && \
        ([regexp ".*Full Moon.*" $newtext])} then {
      set moondata(phase) "Full Moon"
      set moondata_changed 1
    }
    if {(!$moondata(found)) && \
        ([regexp ".*Last quarter.*" $newtext])} then {
      set moondata(phase) "Last quarter"
      set moondata_changed 1
    }
    if {$conf(debug)} then {
      set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] a]
      puts $fd $text
      close $fd
    }
  }

  # check/update moon data once an hour
  proc time:hourly {min hour day month year} {
    variable moondata
    set ctime [ctime [clock seconds]]
    set date "[lrange $ctime 0 2] [lindex $ctime 4]"
    if {(![info exists moondata(date)]) || \
        ([string compare $date $moondata(date)])} then {
      getmoon
    }
    return 0
  }



############################################################

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc time"
    }
    foreach i $what {
      switch -exact -- $i {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}moon ${NamespaceCurrent}::pub:moon
          $action pub m|- ${char}getmoon ${NamespaceCurrent}::pub:getmoon
        }
        msg {
          $action msg -|- moon ${NamespaceCurrent}::msg:moon
          $action msg m|- getmoon ${NamespaceCurrent}::msg:getmoon
        }
        dcc {
          $action dcc -|- moon ${NamespaceCurrent}::dcc:moon
          $action dcc m|- getmoon ${NamespaceCurrent}::dcc:getmoon
        }
        time {
          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:hourly
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    moondatafile moonphase-cache.data
    debug 0
    debugfile moonphase.debug
    host www.usno.navy.mil
    path /cgi-bin/aa_pap
    place "Jasper National Park"
    port 80
    xx0 -1
    xx1 118
    xx2 01
    yy0 1
    yy1 52
    yy2 04
    zz0 -1
    zz1 08
  }

  # Configuration data descriptions
  array set configDataDesc {
    moondatafile "\[file\] file that will store the moon phase data"
    debug "\[0/1\] write to a debug file (this WILL slow things down)"
    debugfile "\[file\] file to write raw html to"
    host "\[host\] http server host"
    path "\[url\] cgi url for post"
    place "(Place Name) name of place"
    port "\[port\] http server port"
    xx0 "(Longitude) \"1\" east, \"-1\" west"
    xx1 "(Longitude) \"xxx\" degrees"
    xx2 "(Longitude) \"xx\" minutes"
    yy0 "(Latitude) \"1\" north, \"-1\" south"
    yy1 "(Latitude) \"yy\" degrees"
    yy2 "(Latitude) \"yy\" minutes"
    zz0 "(Time Zone) \"1\" east of Greenwich, \"-1\" west of Greenwich"
    zz1 "(Time Zone) \"00\" hour"
  }

} ;# namespace moonphase

return
