###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: smtp.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: smtp
# version: 0.5.2.3
# config: smtp.conf
# description: Provides functions other modules can use to send email.
# provides: sendEmail

namespace eval smtp {

  ##
  ## Send an email
  ##
  ## Args:    to, from, subject, body
  ## Returns: nothing
  ## Errors:  if unable to connect to outgoing server
  ##
  proc sendEmail {to from subject body} {
    variable NamespaceCurrent
    variable configData
    variable EmailData

    set EmailData(to) $to
    set EmailData(from) $from
    set EmailData(subject) $subject
    set EmailData(body) $body
    if {[catch {set idx [connect $configData(host) $configData(port)]} result]} then {
      unset EmailData
      error $result
    } else {
      control $idx ${NamespaceCurrent}::control:smtp
    }
    return
  }

  ##
  ## Communicate with sendmail
  ##
  ## Args:    idx, arg
  ## Returns: 1 if message accepted
  ##          0 otherwise
  ##
  proc control:smtp {idx arg} {
    variable EmailData

    switch -exact -- [lindex [split $arg] 0] {
      220 {
        putdcc $idx "MAIL FROM: $EmailData(from)"
      }
      250 {
        switch -exact -- [string tolower [join [lrange [split $arg] 2 end]]] {
          "sender ok" {
            putdcc $idx "RCPT TO: $EmailData(to)"
          }
          "recipient ok" {
            putdcc $idx "DATA"
          }
          "message accepted for delivery" {
            putdcc $idx "QUIT"
            unset EmailData
            return 1
          }
        }
      }
      354 {
        putdcc $idx "Subject: $EmailData(subject)"
        putdcc $idx $EmailData(body)
        putdcc $idx "."
      }
    }
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    host mail
    port 25
  }

  # Configuration data descriptions
  array set configDataDesc {
    host "\[host\] outgoing email server host"
    port "\[port\] outgoing email server port"
  }

} ;# namespace smtp

return
