###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: pointcmds.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: wrpg-pointcmds
# version: 1.0.0.0
# description: Wolf RPG user hit point and food point dcc, msg and public \
#	commands.
# requires: authCheck commandChar isNumber put putnot checkUser findUser \
#	userPointData userStatusData

namespace eval wrpg-pointcmds {

  ##
  ## Increases a user's hp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:addhp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg addhp incr hp]
  }

  ##
  ## Increases a user's hp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:addhp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg addhp incr hp]
  }

  ##
  ## Increases a user's hp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:addhp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg addhp incr hp]
  }

  ##
  ## Decreases a user's hp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:delhp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg delhp decr hp]
  }

  ##
  ## Decreases a user's hp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:delhp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg delhp decr hp]
  }

  ##
  ## Decreases a user's hp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:delhp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg delhp decr hp]
  }

  ##
  ## Change a user's hp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:chhp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg chhp set hp]
  }

  ##
  ## Change a user's hp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:chhp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg chhp set hp]
  }

  ##
  ## Change a user's hp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:chhp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg chhp set hp]
  }

  ##
  ## List a user's hp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:listhp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg listhp list hp]
  }

  ##
  ## List a user's hp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:listhp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg listhp list hp]
  }

  ##
  ## List a user's hp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:listhp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg listhp list hp]
  }

  ##
  ## Increases a user's fp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:addfp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg addfp incr fp]
  }

  ##
  ## Increases a user's fp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:addfp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg addfp incr fp]
  }

  ##
  ## Increases a user's fp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:addfp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg addfp incr fp]
  }

  ##
  ## Decreases a user's fp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:delfp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg delfp decr fp]
  }

  ##
  ## Decreases a user's fp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:delfp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg delfp decr fp]
  }

  ##
  ## Decreases a user's fp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:delfp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg delfp decr fp]
  }

  ##
  ## Change a user's fp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:chfp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg chfp set fp]
  }

  ##
  ## Change a user's fp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:chfp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg chfp set fp]
  }

  ##
  ## Change a user's fp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:chfp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg chfp set fp]
  }

  ##
  ## List a user's fp: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:listfp {nick uhost hand chan arg} {
    return [UserPointData pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg listfp list fp]
  }

  ##
  ## List a user's fp: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:listfp {nick uhost hand arg} {
    return [UserPointData msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg listfp list fp]
  }

  ##
  ## List a user's fp: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:listfp {hand idx arg} {
    return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg listfp list fp]
  }

  ##
  ## Changes/lists a user's hp/fp data
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg, args {command, action, what}
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc UserPointData {type resp1 dest1 resp2 dest2 nick uhost hand chan arg args} {
    set command [lindex $args 0]
    if {[authCheck $type $command $nick $uhost $hand $chan]} then {
      set action [lindex $args 1]
      set what [lindex $args 2]
      set arg [split [string trimright $arg]]
      set who [lindex $arg 0]
      set amount [lindex $arg 1]
      if {![string length $amount]} then {
        set amount 1
      }
      if {(![string length $who]) || (![isNumber $amount])} then {
        switch -exact -- $action {
          list {
            set extra " \[amount\]"
          }
        }
        put $resp2 $dest2 "Usage: [commandChar $type]$command <nick|hand>$extra"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots don't have [string toupper $what]."
            }
          }
        } elseif {![userStatusData $user compare status pack]} then {
          put $resp1 $dest1 "$user is not pack."
        } else {
          if {[set result [userPointData $user $action $what $amount]] != -1} then {
            put $resp1 $dest1 "[string toupper $what] for $user are: $result"
          } else {
            put $resp1 $dest1 "Invalid [string toupper $what] amount: $amount"
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc"
    }
    foreach type $what {
      switch -exact -- $type {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub m|m ${char}addhp ${NamespaceCurrent}::pub:addhp
          $action pub m|m ${char}delhp ${NamespaceCurrent}::pub:delhp
          $action pub m|m ${char}chhp ${NamespaceCurrent}::pub:chhp
          $action pub m|m ${char}listhp ${NamespaceCurrent}::pub:listhp
          $action pub m|m ${char}addfp ${NamespaceCurrent}::pub:addfp
          $action pub m|m ${char}delfp ${NamespaceCurrent}::pub:delfp
          $action pub m|m ${char}chfp ${NamespaceCurrent}::pub:chfp
          $action pub m|m ${char}listfp ${NamespaceCurrent}::pub:listfp
        }
        msg {
          $action msg m|m addhp ${NamespaceCurrent}::msg:addhp
          $action msg m|m delhp ${NamespaceCurrent}::msg:delhp
          $action msg m|m chhp ${NamespaceCurrent}::msg:chhp
          $action msg m|m listhp ${NamespaceCurrent}::msg:listhp
          $action msg m|m addfp ${NamespaceCurrent}::msg:addfp
          $action msg m|m delfp ${NamespaceCurrent}::msg:delfp
          $action msg m|m chfp ${NamespaceCurrent}::msg:chfp
          $action msg m|m listfp ${NamespaceCurrent}::msg:listfp
        }
        dcc {
          $action dcc m|m addhp ${NamespaceCurrent}::dcc:addhp
          $action dcc m|m delhp ${NamespaceCurrent}::dcc:delhp
          $action dcc m|m chhp ${NamespaceCurrent}::dcc:chhp
          $action dcc m|m listhp ${NamespaceCurrent}::dcc:listhp
          $action dcc m|m addfp ${NamespaceCurrent}::dcc:addfp
          $action dcc m|m delfp ${NamespaceCurrent}::dcc:delfp
          $action dcc m|m chfp ${NamespaceCurrent}::dcc:chfp
          $action dcc m|m listfp ${NamespaceCurrent}::dcc:listfp
        }
      }
    }
    return
  }

} ;# namespace wrpg-pointcmds

return
