###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: userdata.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: wrpg-userdata
# version: 1.0.0.0
# description: Wolf RPG shared user status and point data support.
# requires: updateStatusData updatePointData

namespace eval wrpg-userdata {

  ##
  ## Check/update user status and point data hourly
  ##
  ## Args:    minute, hour, day, month, year
  ## Returns: 0
  ##
  proc time:hourly {min hour day month year} {
    updateStatusData
    updatePointData
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable configData
    variable oldConfigData
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what time
    }
    foreach type $what {
      switch -exact -- $type {
        time {
          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:hourly
        }
      }
    }
    return
  }

} ;# namespace wrpg-userdata

return
